package edu.wpi.cs.dsrg.xmldb.xat.common.operator.updateoperator;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;

/**
 * Implementation for the Insert interface.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public class InsertImp extends UpdateOperatorImp implements Insert {

    /** 
     * relativeVariable is the other variable in ordered INSERT
     * as in INSERT $x BEFORE/AFTER $y(relative variable)
     */
    protected String relativeVariable;

    /** PCDATAcontent to for inserting into mixed type elements */
    protected String PCDATAContent;

    /** indication of the order .. BEFORE/AFTER */
    protected int OrderPredicate;

    /** integer indicating update function name (new_attribute() OR new_ref() )*/
    protected int updateFunction = 2;

    /** vector containing the arguments of the function */
    protected String[] funcArgs;

    /**
     * Assigns a node to this operator .
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0   
     */
    public InsertImp(XATNode xnodInit) {
        super(xnodInit);
    }

    /**
     * INSERT <a> </a>
     * In such case on context will be set and the contents <a> </a>
     * will be retrieved from the right child of the insert node     
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit is the context 
     * @since   1.0        
     */
    public InsertImp(XATNode xnodInit, String contextInit) {
        super(xnodInit);
        context = contextInit;
    }

    /**
     * INSERT <a> </a> BEFORE/AFTER $y
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit is the context
     * @param relativeVariableInit String
     * @param order int     
     * @since   1.0        
     */
    public InsertImp(
        XATNode xnodInit,
        String contextInit,
        String relativeVariableInit,
        int order) {
        super(xnodInit);
        context = contextInit;
        relativeVariable = relativeVariableInit;
        OrderPredicate = order;
    }

    /**
    	 * INSERT $x 
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit is the context
     * @param relativeVariableInit String
     * @param order int     
     * @since   1.0        
     */
    public InsertImp(XATNode xnodInit, String contextInit, String variableInit) {
        super(xnodInit);
        context = contextInit;
        variable = variableInit;
    }

    /**
     * INSERT $x BEFORE/AFTER $y 
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit is the context
     * @param relativeVariableInit String
     * @param order int     
     * @since   1.0        
     */
    public InsertImp(
        XATNode xnodInit,
        String contextInit,
        String variableInit,
        String relativeVariableInit,
        int order) {
        super(xnodInit);
        context = contextInit;
        variable = variableInit;
        relativeVariable = relativeVariableInit;
        OrderPredicate = order;
    }

    /** 
     * INSERT new_attribute()/new_ref() 
     * @param xnodInit edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param contextInit is the context
     * @param relativeVariableInit String
     * @param order int     
     * @since   1.0        
     */
    public InsertImp(
        XATNode xnodInit,
        String contextInit,
        int updateFunctionInit,
        String[] params) {
        super(xnodInit);
        context = contextInit;
        updateFunction = updateFunctionInit;
        funcArgs = params;

    }

    /** 
     * Gets the order predicate within the operator
     * @return int
     * @since   1.0        
     */
    public int getOrderPredicate() {
        return OrderPredicate;
    }

    /** 
     * Gets the PCDATA content of the operator 
     * @return int
     * @since   1.0        
     */
    public String getPCDATAContent() {
        return PCDATAContent;
    }

    /** 
     * Gets the right (secondary) data source
     * @return int
     * @thows XATNodeDoesNotExistException
     * @since   1.0        
     */
    public XATTable getRight() throws XATNodeDoesNotExistException {
        try {
            return xnodOwner.getChild(1).getOperator().getOutput();
        } catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
    }

    /** 
     * Gets the relative Variable in the insert statement.
     * @return int
     * @since   1.0        
     */
    public String getRelativeVariable() {
        return relativeVariable;
    }

    /** 
     * Gets the value of updateFunction.
     * @return int
     * @since   1.0        
     */
    public int getUpdateFunction() {
        return updateFunction;
    }

    /**
     * Gets the function arguments  
     * @return an array of String
     * @since   1.0        
     */
    public String[] getFuncArgs() {
        return funcArgs;
    }

    /** 
     * Sets the updateFunction to value that represents type of update Function.
     * @param updateFunc int
     * @since   1.0        
     */
    public void setUpdateFunction(int updateFunc) {
        updateFunction = updateFunc;
    }

    /** 
     * Sets the order predicate of the query (BEFORE/AFTER)
     * @param newOrder int
     * @since   1.0        
     */
    public void setOrderPredicate(int newOrder) {
        OrderPredicate = newOrder;
    }

    /** 
     * Sets the PCDATA content to be inserted
     * @param pcdata String
     * @since   1.0        
     */
    public void setPCDATAContent(String pcdata) {
        PCDATAContent = pcdata;
    }

    /** 
     * Sets the relative Variable in the insert statement
     * @param relVar String
     * @since   1.0        
     */
    public void setRelativeVariable(String relVar) {
        relativeVariable = relVar;
    }

    /** 
     * Sets the function arguments value.
     * @param funArgs is an array String
     * @since   1.0        
     */
    public void setFuncArgs(String[] funArgs) {
        funcArgs = funArgs;
    }
}