package edu.wpi.cs.dsrg.xmldb.xat.common.treestructure;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;

/** 
 * This class will print out the XATTree in an indent format. 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0 
 */

public class XATTreeDefaultVisitor extends InheritableVisitor {
    /** 
     * Identify the current level of visiting. used for indent purpose.
     * @since 1.0
     */
    int level;

    /**
     * The default XATTreeDefaultVisitor constructor.
     * @since 1.0
     */
    public XATTreeDefaultVisitor() {
        // init the current level to 0.
        level = 0;
    }
    public Object dispatch() {
        return null;
    }
    public Object dispatch(Object obj) {
        System.out.println(indent() + obj.toString());
        return null;
    }
    /**
     * Preconstruct an indent buffer as format.
     * @return String The constructed indent buffer.
     * @since 1.0
     */
    String indent() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < level; i++) {
            buf.append("  ");
        }
        return buf.toString();
    }
    /**
     * This method recursively traverse the XATTree to convert it as indent format.
     * @return Object The indent format XATTree.
     * @param tree The XATTree convert for.
     * @since 1.0
     */
    Object visit(XATTree tree) {
        // reset the current level.
        level = 0;
        // output the root.
        visit(tree.getRoot());
        return null;
    }
    /**
     * This method recursively traverse the subtree of the XATNode to convert it as indent format.
     * @return Object The indent format XATTree.
     * @param node The XATNode convert for.
     * @since 1.0
     */   
    Object visit(XATNode node) {
        System.out.println(indent() + node.toString());
        level++;
        XATNode[] children = node.getChildren();
        for (int i = 0; i < children.length; i++) {
            visit(children[i]);
        }
        return null;
    }
}