package edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

/**
 * This class is an implementation of Visitable interface.
 * It is an example of how to use the Element interface in the visitor design pattern. 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0 
 */

public class VisitableImp implements Visitable
{
    /** 
     * This method decides the traverse strategy.
     * @return void
     * @param vi The visitor accepted.
     * @since 1.0
     */
    public void accept(Visitor vi)
    {
        // if this is a simple class without any children,
        // then, just call the visitor to visit itself is good enough.
        // For abstract class, it don't need to visit itself.
        vi.visit(this);
        // traverse the parent attributes if any.
        // if the super class's accept is abstract, then don't need to call it.
        // otherwise add: "super.accept(vi);"
        // but if it is a complex class, then you need to provide
        // the traverse strategy in this accept function.
        // for example, if this is an tree node. you might have something like:

        /*
        for (int i = 0; i < children.length; i++){
          children[i].accept(vi);
        }
        */
    }
}
