package edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration;

import edu.wpi.cs.dsrg.utils.*;
import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import java.util.Vector;
/**
 * IncrementalSQL is stores an SQL query that is incrementally built.
 * @author: Brad Pielech
 */
public class IncrementalSQL extends IncrementalQuery {
    /**select clause  */
    protected Vector select;
    /**position fragment  */
    protected String positionFrag;
    /**from clause */
    protected Vector from;
    /**where clause */
    protected Vector where;
    /**groupBy clause */
    protected Vector groupBy;
    /**orderBy clause */
    protected Vector orderBy;
    /**distinct clause */
    protected Vector distinct;
    /**If there is an aggregate function applied to some column*/
    protected Vector function;

	public static int MULTISTEPORDER = 0;
	public static int SINGLESTEPORDER = 1;
	/** Lists all Table.Column values and their equivalent Rename Values*/
	protected java.util.Hashtable renameHash;
	/** Lists all Rename values and their equivalent Table.Column Values*/
	protected java.util.Hashtable tableColHash;
    public IncrementalSQL(BindingTable varTable) {
		super(varTable);
	    select = new Vector();
        from = new Vector();
        where = new Vector();
        groupBy = new Vector();
        orderBy = new Vector();
        distinct = new Vector();
        function = new Vector();

        positionFrag = new String();
    }

    public void addClause(QueryFragment fragment) {

        //create a copy of fragment
        QueryFragment newFrag = (QueryFragment) fragment.clone();
        switch (newFrag.getClauseType()) {
            case QueryTypes.SELECT :
                {
                    select.addElement(newFrag);
                    break;
                }
            case QueryTypes.FROM :
                {
	                String newBinding = newFrag.getBinding();
	                //before adding the from, make sure that the binding for this new table isnt already there
	                for (int i = 0; i < from.size(); i++){
	                	QueryFragment q1 = (QueryFragment)from.elementAt(i);
	                	String oldBinding = q1.getBinding();
	                	if(oldBinding.equals(newBinding)){
							return;
	                	}
	                }
                    from.addElement(newFrag);
                    break;
                }
            case QueryTypes.DISTINCT :
                {
                    distinct.addElement(newFrag);
                    break;
                }
            case QueryTypes.FUNCTION :
                {
                    function.addElement(newFrag);
                    break;
                }

            case QueryTypes.WHERE :
                {
                    where.addElement(newFrag);
                    break;
                }
            case QueryTypes.ORDERBY :
                {
                    orderBy.addElement(newFrag);
                    break;
                }
            case QueryTypes.GROUPBY :
                {
                    groupBy.addElement(newFrag);
                    break;
                }

            default :
                { //this happens only with Source operators that wont be converted so SQL, nothing happens though

                }
        }

    }
/**
 * Insert the method's description here.
 * Creation date: (4/8/2003 1:34:01 PM)
 * @return java.lang.String
 * @param params java.lang.String
 * @param type int
 */
public String createOrder(String params, int type) {
    //find the multisteporder function (because the binding table is dummy)
    // and replace it with a character, ie
    // function_multisteporder = 12
    //  becomes
    // a = 12

    //first make subquery
    //function.removeElementAt(k);

    StringTokenizer paramToks = new StringTokenizer(params, ";");
    String dxvPath = "";
    String state = "";
    String context = "";
    String aorn = "";

    for (int m = 0; m <= 3; m++) {
        String tok = paramToks.nextToken();
        switch (m) {
            case 0 :
                {
                    dxvPath = tok;
                    break;
                }
            case 1 :
                {
                    state = tok;
                    break;
                }
            case 2 :
                {
                    context = tok;
                    break;
                }
            case 3 :
                {
                    aorn = tok;
                    break;
                }
        }
    }

    //Take context, and make it "SQL" like
    // EDGES/EDGES.ROWS/target ~ EDGES.target
    // CONTEXT is stored in this pattern: PID1:POS1:PID2:POS2:PID3
    //  where PID1 is the root most PID context required by the view Query, and is equivalent to EDGES/EDGES.ROW/PID
    //  PID3 is the last PID required by the view query to group over
    Vector conVec = new Vector();
    //if it is fully preservered, just add the final most PID and POS
    if(state.startsWith("fully")){
	    conVec.add(context);
	    conVec.add(dxvPath);
    }else{
	    //Otherwise, add all of the inner PIDs and POSs followed by the outermoust PID and POS
	    StringTokenizer contextToks = new StringTokenizer(context, ":");
    	while (contextToks.hasMoreTokens()) {
        	conVec.add(contextToks.nextToken());
    	}
	    conVec.add(dxvPath);
	}

    // determine whether numeric codings are used or not
    // this is useful for context
    boolean tonumber = false;
    if (aorn.trim().equalsIgnoreCase("N")) {
        tonumber = true;
    }

    //first value is context
    //Get all context order values for ordering, the PID is assumed to be numeric
    String contextVar = new String("");
    // Some XML fragments may map to more than one key, such as 
    //  $songMap := /EDGES/EDGES.ROW/SOURCE
    //  $playMap := /EDGES/EDGES.ROW/SOURCE
    // Hence a vector of keys is returned, and a precaution must be made
    //	to order and partition over the same table.
    boolean fewTables = false;
    String tableToMatch = new String();
    if (type == SINGLESTEPORDER) {
        for (int o = 0; o < conVec.size(); o = o + 2) {
			String element = (String)conVec.elementAt(o);
	        Vector keys = getVariableTable().getKeysByXMLValue("/SOURCE" + element);
	        
	        String innerConVar = (String)keys.elementAt(0);
            String val = (String) renameHash.get(innerConVar);
	        if(keys.size() > 1){
		        fewTables = true;
		        tableToMatch = val.substring(0, val.indexOf("."));
	        }
	        
            contextVar += " to_number(" + val + "),";
        }
        //remove final ','
        if (!contextVar.equals("")) {
            contextVar = contextVar.substring(0, contextVar.length() - 1);
        }

    }
    
    //Second value is order
    String orderVar = new String("");
    // user every other POS element for ordering
    for (int n = 1; n < conVec.size(); n = n + 2) {
	    String element = (String)conVec.elementAt(n);
	    String innerConVar = new String(); String val = new String();
	    if(fewTables){
		    Vector keys = getVariableTable().getKeysByXMLValue("/SOURCE" + element);
	        for(int j =0; j<keys.size() && !innerConVar.startsWith(tableToMatch);j++){
				innerConVar = (String)keys.elementAt(0);
	        }
            val = (String) renameHash.get(innerConVar);

	    }else{
	    	innerConVar = getVariableTable().getKeyByXMLValue("/SOURCE" + element);
        	val = (String) renameHash.get(innerConVar);
	    }
        if (tonumber) {
        	orderVar += " to_number(" + val + "),";
        }else{
	        orderVar += val + ",";
        }
    }
    //remove final ','
    if (!orderVar.equals("")) {
        orderVar = orderVar.substring(0, orderVar.length() - 1);
    }

  
    //Put it all together
    String selectMore = ", row_number() over ";
    String partition = new String();
    if (type == SINGLESTEPORDER) {
        partition = " partition by " + contextVar;
    } else {
        partition = "";
    }
    String orderby = " order by " + orderVar + " ";


    String orderReturn = selectMore + " (" + partition + orderby;
    //return string here
    return orderReturn;

}
/**
* getSQLStatement returns 1 SQL statement that is the combination
* of the various clause properties
* @return String
*/
public String getStatement() throws SQLNoStatementExists {

    //make sure the statement has a from clause, otherwise it is invalid
    if (from.size() == 0 || select.size() == 0) {
        throw new SQLNoStatementExists();
    }

    BindingTable sqlBindTable = getVariableTable();

    //each Vector with associated items will get added to these
    // strings in an SQL manner
    String selectFrag = new String("");
    String fromFrag = new String("");
    String whereFrag = new String("");
    String groupbyFrag = new String("");
    String orderbyFrag = new String("");

    //at the end, all the above strings should be combined to form
    //  the desired sql statement (may be combined differently for
    //  some specific queries
    String sqlReturn = new String("");

    boolean pos = false;

    QueryFragment s1, f1;
    selectFrag = "SELECT ";
    //if there is a distinct fragment, add the keyword to the Select clause
    /*
    if (distinct.size() > 0) {
        selectFrag += "DISTINCT ";
    }
    */
    renameHash = new Hashtable();
    tableColHash = new Hashtable();
    //before adding the column, check to see if it should be distinct or have a function around it
    for (int i = 0; i < select.size(); i++) {
        s1 = (QueryFragment) select.elementAt(i);
        String binding = s1.getBinding();
        String selectName = s1.getName();
        //if we find a matching function, then dont add it later
        boolean funLookUp = false;
        /**
        * look over the functions and try to match the Select's binding to the parameter of the function
        * function name = function.getName
        * function params = function.getBinding()
        */
        for (int k = 0; k < function.size(); k++) {
            f1 = (QueryFragment) function.elementAt(k);
            if (!f1.getName().toUpperCase().equals("SINGLESTEPORDER")
                && !f1.getName().toUpperCase().equals("MULTISTEPORDER")) {
                String params = f1.getParams();
                //drop out the table binding ("table")
                int lastIndex = selectName.lastIndexOf(".") + 1;
                String columnName = selectName.substring(lastIndex, selectName.length());
                //if this function's parameters contain this binding
                if (params.indexOf(columnName) > 0) {
                    selectFrag += f1.getName().toUpperCase()
                        + "("
                        + s1.getName()
                        + ") AS \""
                        + f1.getBinding()
                        + "\", ";
                    funLookUp = true;
                }
                renameHash.put(f1.getBinding(), s1.getName());
                tableColHash.put(s1.getName(), f1.getBinding());
                //function.removeElementAt(k);
            }

        }
        //we found a function matching, so the statement is already added
        // else, add it now, as long as it isn't the TABLE's name, as denoted by "s..."
		// possible HACK HERE 04/08/2003
		// ALSO, if the select name already exists, which could be the case with
		//	a LOJ with identical navigation routes under it, then don't add it again
        if (!funLookUp && !selectName.startsWith("\"s") && selectFrag.indexOf(s1.getBinding()) < 0) {
            selectFrag += s1.getName() + " AS \"" + s1.getBinding() + "\", ";
            renameHash.put(s1.getBinding(), s1.getName());
            tableColHash.put(s1.getName(), s1.getBinding());
        }
    }
    //trim the last comma
    selectFrag = selectFrag.substring(0, (selectFrag.length() - 2));
    fromFrag = " FROM ";
    for (int i = 0; i < from.size(); i++) {
        s1 = (QueryFragment) from.elementAt(i);
        fromFrag += s1.getName() + " \"" + s1.getBinding() + "\", ";
    }

    //trim the last comma
    fromFrag = fromFrag.substring(0, (fromFrag.length() - 2));

    //if there is a where clause, then add it
    //WHERE, ORDERBY, and GROUPBY just have columns names or expressions so there is no binding
    if (where.size() > 0) {
        whereFrag = " WHERE ";
        for (int i = 0; i < where.size(); i++) {
            s1 = (QueryFragment) where.elementAt(i);
            String whereClause = s1.getName();
            //if the where fragment already exists, as in the previous case
            //	of navigates -- select in a LOJ, don't create it again.
            if(whereFrag.indexOf(s1.getName()) < 0)
	            whereFrag += whereClause + " AND ";
        }
        //trim the last AND
        whereFrag = whereFrag.substring(0, (whereFrag.length() - 4));
    }

    /**
    * Add any function not already in the SELECT clause to WHERE clause
    * function name = function.getName
    * function params = function.getBinding()
    */
    for (int k = 0; k < function.size(); k++) {
        f1 = (QueryFragment) function.elementAt(k);
        String params = f1.getParams();
        if (f1.getName().equalsIgnoreCase("multisteporder")) {
	        pos = true;
            String newFromFrag =
                " FROM (" + selectFrag + createOrder(params, MULTISTEPORDER);
            newFromFrag += ") \"" + f1.getBinding() + "\"" + fromFrag;
            fromFrag = newFromFrag;
			renameHash.put(f1.getBinding(), "multisteporder");
            //
            //If it is single step order...
            //
        } else
            if (f1.getName().equalsIgnoreCase("singlesteporder")) {
	            pos = true;
                String newFromFrag =
                    " FROM (" + selectFrag + createOrder(params, SINGLESTEPORDER);
                newFromFrag += ") \"" + f1.getBinding() + "\""+ fromFrag;
                fromFrag = newFromFrag;
                renameHash.put(f1.getBinding(),"singlesteporder");
            }

    }

    if (orderBy.size() > 0) {
        orderbyFrag = " ORDER BY ";
        for (int i = 0; i < orderBy.size(); i++) {
            s1 = (QueryFragment) orderBy.elementAt(i);
            orderbyFrag += s1.getName() + ", ";

        }
        //trim the last comma
        orderbyFrag = orderbyFrag.substring(0, (orderbyFrag.length() - 2));

    }

    if (groupBy.size() > 0) {
        groupbyFrag = " GROUP BY ";
        for (int i = 0; i < groupBy.size(); i++) {
            s1 = (QueryFragment) groupBy.elementAt(i);
            groupbyFrag += s1.getName() + ", ";
        }
        //trim the last comma
        groupbyFrag = groupbyFrag.substring(0, (groupbyFrag.length() - 2));
    }

    String myNewWhere = toColName(whereFrag);
    //add getcolumns to outermost select, nest it
    if(!pos)
    	sqlReturn = stripColumns(selectFrag) + " FROM (" + selectFrag + fromFrag + ")" + whereFrag + orderbyFrag + groupbyFrag;
	else
		sqlReturn = stripColumns(selectFrag) + fromFrag + myNewWhere + ") WHERE " + positionFrag + orderbyFrag + groupbyFrag;

    return sqlReturn;

}
    /**
     * ReInitialize's this IncrementalStatement
     * 
     */
    public void initialize() {

        select = new Vector();
        from = new Vector();
        where = new Vector();
        groupBy = new Vector();
        orderBy = new Vector();
        distinct = new Vector();
        function = new Vector();
    }
/**
 * This method will strip the original column element from the Select
 *  leaving the renamed value
 *	string so that it can be used appropriately the outmost nest:
 *		"s100004".EDGES AS "$col220001"
 *			to
 *		"$col220001"
 *
 * The input will resemble "SELECT [DISTINCT]? "blue"."blee" as "blah"
 * The output will resemble "SELECT [DISTINCT]? "blah""
 * Creation date: (4/8/2003 2:14:35 PM)
 * @return java.lang.String
 * @param input java.lang.String
 */
public String stripColumns(String input) {
    String output = new String();
    String head = input.substring(0, input.indexOf("\""));
    String body = input.substring(input.indexOf("\""), input.length());
    try {
        while (body.length() > 0) {
            org.apache.regexp.RE reg =
                new org.apache.regexp.RE(
                    "\"\\$?[$a-zA-Z0-9_]*\".[a-zA-Z0-9_]* AS (\"\\$[a-zA-Z0-9_]*\")");
            boolean matched = reg.match(body);
            if (matched) {
	            String wholeMatch = reg.getParen(0);
                String column = reg.getParen(1);
                if((body.length() - wholeMatch.length())<= 0){
	                body ="";
                }else{
	            	body = body.substring(wholeMatch.length()+2, body.length());
                }
                output += column + ", ";
            }else{
	            body = "";
            }
        }
    } catch (Exception e) {
        e.printStackTrace();
    }
    if (!output.equals("")) {
        //remove the last ','
        output = output.substring(0, output.length() - 2);
    }

    output = head + output;
    return output;
}
/**
 * This method will strip the renaming elements from the Select
 *	string so that it can be used appropriately in nested queries:
 *		"s100004".EDGES AS "$col220001"
 *			to
 *		"s100004".EDGES
 *
 * The input will resemble "SELECT [DISTINCT]? "blue"."blee" as "blah"
 * The output will resemble "SELECT [DISTINCT]? "blue"."blee""
 * Creation date: (4/7/2003 2:14:35 PM)
 * @return java.lang.String
 * @param input java.lang.String
 */
public String stripRename(String input) {
    String output = new String();
    String head = input.substring(0, input.indexOf("\""));
    String body = input.substring(input.indexOf("\""), input.length());
    try {
        while (body.length() > 0) {
            org.apache.regexp.RE reg =
                new org.apache.regexp.RE(
                    "(\"\\$?[$a-zA-Z0-9]*\".[a-zA-Z0-9]*) AS \"\\$[a-zA-Z0-9]*\"");
            boolean matched = reg.match(body);
            if (matched) {
	            String wholeMatch = reg.getParen(0);
                String column = reg.getParen(1);
                if((body.length() - wholeMatch.length())<= 0){
	                body ="";
                }else{
	            	body = body.substring(wholeMatch.length()+2, body.length());
                }
                output += column + ", ";
            }
        }
    } catch (Exception e) {
        e.printStackTrace();
    }
    if (!output.equals("")) {
        //remove the last ','
        output = output.substring(0, output.length() - 2);
    }

    output = head + output;
    return output;
}
/**
 * This method will translate the rename element to the Column Name element
 *		"$col220001"
 *			to
 *		"s100004".EDGES
 *
 * The input will resemble "WHERE "blah" = ..."
 * The output will resemble "WHERE "blee.bloo" = ..."
 * Creation date: (4/8/2003 2:14:35 PM)
 * @return java.lang.String
 * @param input java.lang.String
 */
public String toColName(String input) { 
  	String output = new String();
    String head = input.substring(0, input.indexOf("\""));
    String body = input.substring(input.indexOf("\""), input.length());
    String bodyPiece = new String();
    try {
        while (body.length() > 0) {
            org.apache.regexp.RE reg =
                new org.apache.regexp.RE(
                    "\"(\\$[a-zA-Z0-9]*)\"");
            boolean matched = reg.match(body);
            if (matched) {
	            String wholeMatch = reg.getParen(0);
                String column = reg.getParen(1);
                //find it's table.column equivalent
                String tableCol = (String)renameHash.get(column);
                if((body.length() - wholeMatch.length())<= 0){
	                body ="";
                }else{
	            	body = body.substring(wholeMatch.length(), body.length());	            		            	
	            	if(body.indexOf("\"") < 0){
		            	//if no more "renames" are left, just append the whole thing as bodyPiece
						bodyPiece = body;
						body = "";
	            	}else{
	            	//find up to the next double quote, ignore single quote and others as text
	            	//	not rename column names
		            	bodyPiece = body.substring(0,body.indexOf("\""));
		            	body = body.substring(body.indexOf("\""), body.length());
                	}   	
                }

                if(!tableCol.equals("singlesteporder") && !tableCol.equals("multisteporder")){
	                output += tableCol + bodyPiece ;
                }else{
	                //use original column, not "singlesteporder" or "multisteporder"
	                positionFrag +=  "\"" + column + "\"" + bodyPiece;
                }
            }
        }
    } catch (Exception e) {
        e.printStackTrace();
    }
    
    if (output.endsWith("AND ") || output.endsWith("AND")) {
        //remove the last ','
        output = output.substring(0, output.lastIndexOf("AND"));
    }

    output = head + output;
    return output;
}
/**
 * Insert the method's description here.
 * Creation date: (9/17/2002 3:40:51 PM)
 * @return java.lang.String
 */
public String toString() {
	try {
		return getStatement();
	} catch (SQLNoStatementExists e1) {
		return super.toString();
	}
}
}
