package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.*;
import java.lang.reflect.*;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;

/**
 * This Class is used to optimize the orderby operators ( if there is any in given XAT ).
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */
public class ReWriteOrderByRules extends ReWriteRules {
/**
 * Default ReWriteOrderByRules constructor.
 * @since 1.0
 */
public ReWriteOrderByRules() {
	super();
}
/**
 * ReWriteOrderByRules constructor.
 * @param sourceList The list of operator types being considered to apply this rewrite rule.
 * @since 1.0   
 */

public ReWriteOrderByRules(Vector sourceList) {
	super(sourceList);
}
/**
 * ReWriteOrderByRules constructor.
 * Constructs a CancelOutTraversal object using XAT and Rewrite Rules and order.
 * @param sourceList The list of operator types being considered to apply this rewrite rule.
 * @param varList java.util.Hashtable, the binding table of given XAT.
 * @since 1.0   
 * @see   BindingTable
 */

public ReWriteOrderByRules(Vector sourceList, BindingTable varList) {
	super(sourceList, varList);
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (NameColumn).
 * @since 1.0   
 * @see OrderBy
 * @see NameColumn
 * @see ReWriteResult
 */
 
public Object visit(OrderBy parent, NameColumn child) {
    ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Source).
 * @since 1.0   
 * @see OrderBy
 * @see Source
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, Source child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (GroupBy).
 * @since 1.0   
 * @see OrderBy
 * @see GroupBy
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, GroupBy child) {
    ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Join).
 * @since 1.0   
 * @see OrderBy
 * @see Join
 * @see ReWriteResult
 */
 
public Object visit(OrderBy parent, Join child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (OrderBy).
 * @since 1.0   
 * @see OrderBy
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, OrderBy child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Project).
 * @since 1.0   
 * @see OrderBy
 * @see Project
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, Project child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Select).
 * @since 1.0   
 * @see OrderBy
 * @see Select
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, Select child) {
    //All the action should be here
    //Push it up to the select, if the select and the orderby have the same column condition
    // stop there, otherwise continue.

    return null;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Union).
 * @since 1.0   
 * @see OrderBy
 * @see Union
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, Union child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Aggregate).
 * @since 1.0   
 * @see Aggregate
 * @see ReWriteResult
 * @see OrderBy
 */

public Object visit(OrderBy parent, Aggregate child) {
    ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
/**
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent XAT operator (OrderBy).
 * @param child The child XAT Operator (Tagger).
 * @since 1.0   
 * @see OrderBy
 * @see Tagger
 * @see ReWriteResult
 */

public Object visit(OrderBy parent, Tagger child) {
	ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);
    p1.setSwapped(false);
    return p1;
}
}
