package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

/**
 * This Class is used to rewrite the XAT using position rules.
 * <pre>
 * Result object created when a ReWriteRule is evaluated
 * The parent and child node refer to the two nodes that were evaluated in
 * the equivalance rule.  
 * <pre>
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 * @see ReWriteRules
 * @see ReWriteNavigationRules
 * @see ReWriteOrderByRules 
 */

public class ReWriteResult {

    /**
     * Boolean value to indicate if two operators (parent & child) should be swapped or not.
     * True if the nodes were swapped, false otherwise.
     * @since 1.0
     */
    private boolean swapped;
    /**
     * Boolean value to indicate if the parent operator need to be swapped further or not.
     * True if parent nodes is in its final position, false otherwise.
     * @since 1.0
     */
    private boolean finalPosition;
    /**
     * Boolean value to indicate if the parent operator should be deleted or not.
     * True if the parent node should be deleted, false otherwise
     * @since 1.0
     */
    private boolean deleteParentNode;
    /**
     * Boolean value to indicate if the child operator should be deleted or not.
     * True if the child node should be deleted, false otherwise
     * @since 1.0
     */
    private boolean deleteChildNode;
    /**
     * Boolean value to indicate if the parent operator was moved or not.
     * True if the parent node was moved.
     * @since 1.0
     */
    private boolean moved;

/**
 * Default ReWriteResult constructor.
 * @since 1.0   
 */

public ReWriteResult() {
    swapped = false;
    finalPosition = false;
    deleteParentNode = false;
    deleteChildNode = false;
    moved = false;
}
/**
 * Get the value of attribute deleteChildNode.
 * @return boolean Indicate whether the child node should be deleted.
 * @since  1.0   
 */

public boolean isDeleteChildNode() {
    return deleteChildNode;
}
/**
 * Get the value of attribute deleteParentNode.
 * @return boolean Indicate whether the parent node should be deleted.
 * @since  1.0   
 */

public boolean isDeleteParentNode() {
    return deleteParentNode;
}
/**
 * Get the value of attribute finalPosition.
 * @return boolean Indicate if the node is in its final position after the rule was executed.
 * @since  1.0   
 */

public boolean isFinalPosition() {
    return finalPosition;
}
/**
 * Get the value of attribute moved.
 * @return boolean Indicate if the parent is moved.
 * @since  1.0   
 */

public boolean isMoved() {
    return moved;
}
/**
 * Get the value of attribute swapped.
 * @return boolean Indicate if the nodes were swapped.
 * @since  1.0   
 */

public boolean isSwapped() {
    return swapped;
}
/**
 * Set the attribute of deleteChildNode.
 * @return void 
 * @param newDeleteChildNode Indicate whether the child node should be deleted. 
 * @since  1.0   
 */

public void setDeleteChildNode(boolean newDeleteChildNode) {
    deleteChildNode = newDeleteChildNode;
}
/**
 * Set the attribute of deleteParentNode.
 * @return void 
 * @param newDeleteNode Indicate whether the parent node should be deleted. 
 * @since  1.0   
 */

public void setDeleteParentNode(boolean newDeleteNode) {
    deleteParentNode = newDeleteNode;
}
/**
 * Set the attribute of  finalPosition.
 * @return void 
 * @param newPushedDown Indicate whether the parent node in its final postion
 * @since  1.0   
 */

public void setFinalPosition(boolean newPushedDown) {
    finalPosition = newPushedDown;
}
/**
 * Set the attribute of moved.
 * @return void 
 * @param newMoved Indicate whether the parent node was moved.
 * @since  1.0   
 */

public void setMoved(boolean newMoved) {
    moved = newMoved;
}
/**
 * Set the attribute of swapped.
 * @return void 
 * @param newSuccessful Indicate whether the parent and child nodes should be swapped.
 * @since  1.0   
 */

public void setSwapped(boolean newSuccessful) {
    swapped = newSuccessful;
}
}
