package edu.wpi.cs.dsrg.xmldb.xat.testfiles;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.Vector;
import java.io.*;
import xacute.quilt.*;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.querymerger.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xattreegeneration.*;

/**
 * This Class is used to test the Rainbow Core.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class CombinedJBTest {
/**
 * The default CombinedJBTest constructor.
 * @since 1.0
 */
public CombinedJBTest() {
	super();
}
/**
 * This method is the main function for RainbowCore.
 * The suggested usage is as follows:
 * eg:	two RainbowCore\widm\0user.qlt RainbowCore\widm\view.qlt gen decor rewrite merge schema exec debug sql
 * eg:	one RainbowCore\widm\0user.qlt gen decor rewrite schema exec debug sql 
 *
 * The meaning of flags is as follows:
 * - The first flag should be a string, "one" or "two".  This will show how many queries there are. 
 * Read in either one or two queries, generate trees, then do the following with the remaining FLAGS
 * - decor: decorrelate the query
 * - merge: if there are two trees merge them
 * - rewrite: pushdown the computation
 * - schema: clean up the schema
 * - sql: generate sql
 * - exec: execute the tree
 * - newProp: create new default property values 
 * - debug: debug the entire system
 * - time: shows time statements for each step
 *
 * @since 1.0
 */
 
public static void main(String args[]) throws Exception {

    /***
    * Read in two queries, the first is the mapping, the 2nd is the user.  
    * Call tree gen and decorr on both of them and then merge them before doing pushdown
    */
    //reads a query from a file and translates it
    //into a string

    try {
        String query = null;
        query = readQuery(args[0]);
        QuiltParser parser = new QuiltParser();
        QuiltQuery q = null;
        q = parser.parseQuery(query);
        ParserTraverse pt = new ParserTraverse();
        System.out.println("*************Generating the Mapping Tree****************");
        /*************  Step 1: Generate the Mapping Tree **************/
        pt.visit(q);

        XATTree mappingTree = pt.getTree();
        Debugger mappingDebug = new Debugger(mappingTree);

        /*************  Step 2: Decorrelate the Mapping Tree **************/
        System.out.println(
            "**********Starting Mapping Query Decorrelation************");
        QueryDecorrelation qd1 = new QueryDecorrelation(mappingTree);

        mappingTree = qd1.getDecorTree();
        mappingDebug.printTree();
        System.out.println("\n\n\n");
        System.out.println(
            "%%%%%%%%%%%%%%%%%%%%%%%%%%% Now working with User Query %%%%%%%%%%%%%");
        /**&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
        //********* now do the same for user query */
        query = readQuery(args[1]);
        //parser = new QuiltParser();
        q = parser.parseQuery(query);
        pt = new ParserTraverse();
        /*************  Step 1: Generate the User Tree **************/
        System.out.println("**********Starting User Query Generation************");
        pt.visit(q);
        XATTree userTree = pt.getTree();
        Debugger userDebug = new Debugger(userTree);

        /*************  Step 2: Decorrelate the User Tree **************/
        System.out.println("**********Starting User Query Decorrelation************");
        qd1 = new QueryDecorrelation(userTree);
        userTree = qd1.getDecorTree();
        userDebug.printTree();

        /*********** Now merge the trees *******/
        System.out.println("**********Now merging the trees************");
        MergeVisitor m1 = new MergeVisitor(userTree, mappingTree);
        m1.mergeTrees();
        System.out.println("The user query:");
        userDebug.printTree();
        //System.out.println("The mapping query:");
        //mappingDebug.printTree();

        /*************  Step 3: Pushdown the Tree **************/
        //going to link that tree with the rewrite class
        //first create a vector of the rule classes to look at
        Vector classList = new Vector();
        String s1 =
            new String("edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ReWriteNavigateRules");
        String s2 =
            new String("edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ReWriteSelectRules");
        classList.addElement(s1);
        classList.addElement(s2);

        //instantiate the rules class
        ReWriteRules rules = new ReWriteRules(classList);

        //now make the order of rules to look at
        Vector order = new Vector();
        order.addElement(
            "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavigateImp");
        order.addElement(
            "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.SelectImp");
        //set the variable table in the rules class to be the hashtable from PT
        rules.setVariableTable(pt.getTree().getBindingTable());

        //create the traversal strategy
        /*
        ReWriteTraversal traverse = new ReWriteTraversal(tree, rules, order);
        
        System.out.println("**********Starting pushDown************");
        traverse.doPushdown(traverse.DOWN);
        System.out.println("Current time is:" +  System.currentTimeMillis());
        debug.printTree();
        */

        /*************  Step 4: Generate the SQL **************/
        //pass in the binding hashtable to the sql generator
        /*
        QueryGenerator sqlGen = new QueryGenerator(pt.getTree().getBindingTable());
        System.out.println("**********Starting SQL Generation************");
        sqlGen.generateQuery(tree, sqlGen.FULL, QueryTypes.SQL);
        System.out.println("Current time is:" +  System.currentTimeMillis());
        debug.printTree();
        
        /*************  Step 5: Compute The Schema **************/
        /*
         System.out.println("**********Starting Schema Computation************");
        
        SchemaComputer sc = new SchemaComputer();
        sc.compute(tree);
        
        /*************  Step 6: Execute The Query **************/
        /*
         System.out.println("**********Starting Query Execution ************");
        
        
        QueryExecutor qe = new QueryExecutor("jdbc:oracle:thin:@wpi.wpi.edu:1521:cs", "alex", "alex");
        XATTable resultTable = qe.execute(tree);
        System.out.println("Current time is:" +  System.currentTimeMillis());
        
        System.out.println("************This is the XML Document***********\n");
        System.out.println(resultTable.getColumn(resultTable.getNumberOfColumns()).toString());
        */
    } catch (Exception e) {
        e.printStackTrace(System.out);
        System.exit(-1);
    }

    System.exit(0);

}
/**
 * Reads in the contents of a file and converts it to a string object.
 * @return String The content of the file.
 * @param file The name of file opening.
 * @since 1.0
 */
private static String readQuery(String file) throws Exception {
    BufferedReader in =
        new BufferedReader(new InputStreamReader(new FileInputStream(file)));
    StringBuffer buf = new StringBuffer();
    String s;
    while ((s = in.readLine()) != null)
        buf.append(s + "\n");
    in.close();
    return buf.toString();
}
}
