package edu.wpi.cs.dsrg.xmldb.xat.testfiles;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.*;
import java.util.Vector;
import java.io.*;
import xacute.quilt.*;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.operatorexec.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.querymerger.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xattreegeneration.*;

/**
 * This Class is used to test the Rainbow Core.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class TestSXE {
	/**
	 * The parameters from command line.
	 * @since 1.0
	 */
    public static boolean decor = false;
    public static boolean newProp = false;
    public static boolean merge = false;
    public static boolean cut = false;
    public static boolean rewrite = false;
    public static boolean time = false;
    public static boolean schema = false;
    public static boolean sql = false;
    public static boolean exec = false;
    public static boolean debug = false;
    public static boolean nooutput = false;
    protected final static String PROPERTIES_FILE_NAME = "properties.txt";

   	/**
	 * The variables to record the usage of time.
	 * @since 1.0
	 */
    protected static long timeBegin;
    protected static long timeGen;
    protected static long timeMerge;
    protected static long timeDecor;
    protected static long timeSchema;
    protected static long timeRewrite;
    protected static long timeSQL;
    protected static long timeExec;
    protected static long timeCut;
    protected static long timeTotal;

    /**
     * The parameter used for database connection
     * @since 1.0
     */
    protected static java.lang.String username;
    protected static java.lang.String password;
    protected static java.lang.String uri;

    /**
     * The Properties setting in running the main function.
     * @since 1.0
     */
    protected static Properties defaultProperties;
    protected static java.lang.String metadataTable;
/**
 * The default TestSXE constructor.
 * @since 1.0
 */
public  TestSXE() {
	super();
	timeTotal = 0;
	timeGen = 0;
	timeDecor = 0;
	timeMerge = 0;
	timeRewrite =0;
	timeSchema=0;
	timeSQL=0;
	timeExec=0;
	timeBegin=0;
	timeCut=0;
}
/**
 * Get the default properties from propertity_file.
 * @return java.util.Properties The default properity for main method.
 * @since 1.0
 */
public java.util.Properties getDefaultProperties() {
	return defaultProperties;
}
/**
 * Get the metadataTable.
 * @return String The matadata table.
 * @since 1.0
 */
public static java.lang.String getMetadataTable() {
	return metadataTable;
}
/**
 * Get password of JDBC connection.
 * @return String The user password.
 * @since 1.0
 */
public static java.lang.String getPassword() {
	return password;
}
/**
 * Get the JDBC connection string.
 * @return java.lang.String The JDBC connection string to database.
 * @since 1.0
 */
public static java.lang.String getUri() {
	return uri;
}
/**
 * Get the username for JDBC connection.
 * @return java.lang.String The user name for build the JDBC connection to database.
 * @since 1.0
 */
public static java.lang.String getUsername() {
	return username;
}
/**
 * This method is the main function for RainbowCore.
 * The suggested usage is as follows:
 * eg:	two RainbowCore\widm\0user.qlt RainbowCore\widm\view.qlt gen decor rewrite merge schema exec debug sql
 * eg:	one RainbowCore\widm\0user.qlt gen decor rewrite schema exec debug sql 
 *
 * The meaning of flags is as follows:
 * - The first flag should be a string, "one" or "two".  This will show how many queries there are. 
 * Read in either one or two queries, generate trees, then do the following with the remaining FLAGS
 * - decor: decorrelate the query
 * - merge: if there are two trees merge them
 * - rewrite: pushdown the computation
 * - schema: clean up the schema
 * - sql: generate sql
 * - exec: execute the tree
 * - newProp: create new default property values 
 * - debug: debug the entire system
 * - time: shows time statements for each step
 *
 * @since 1.0
 */

public static void main(String args[]) throws Exception {
    String userQuery = new String();
    String viewQuery = new String();

    /** set which operator factory to use */
    /** this is to generate persistent operators */
    System.setProperty(
        edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.OperatorFactory.factoryID,
        "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.PersistentOperatorFactory");
    /** this generates stream operators (not yet implemented) */
    //System.setProperty(OperatorFactory.factoryID, "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.StreamOperatorFactory");

    timeTotal = 0;
    XATTree userTree = null, mappingTree = null;

    userQuery = readQuery(args[1]);
    System.out.println("User Query: " + args[1]);
    if (args[0].equals("two")) {
        viewQuery = readQuery(args[2]);
        System.out.println("View Query: " + args[2]);
        merge = true;
    }

    int i = 2;
    if (merge) {
        i = 3;
    }

    readParameters(args, i);

    if (debug) {
        // set system properties to debug the Rainbow Core.
        System.setProperty("DEBUG_RewriteUtilities", "OK");
        System.setProperty("DEBUG_DeepCopyMaker", "OK");
        System.setProperty("DEBUG_NavTaggerCancelout", "OK");
        System.setProperty("DEBUG_XATTreeImp", "OK");
        System.setProperty("DEBUG_XATTableImp", "OK");
        System.setProperty("DEBUG_ReWriteTraversal", "OK");
        System.setProperty("DEBUG_ParserTraverse", "OK");
        System.setProperty("DEBUG_SchemaCleanUp", "OK");
    }

    if (newProp && args[5] != null) {
        if (merge) {
            setUsername(args[3]);
            setPassword(args[4]);
            setUri(args[5]);
            setMetadataTable(args[6]);
        } else {
            setUsername(args[2]);
            setPassword(args[3]);
            setUri(args[4]);
            setMetadataTable(args[5]);
        }
        saveProperties(getUsername(), getPassword(), getUri(), getMetadataTable());
        setDefaultProperties();
    } else {
        setUsername("xml");
        setPassword("xml");
        setUri("jdbc:oracle:thin:@dsrg.wpi.edu:1521:ORCL");
        setMetadataTable("RECORDLIST_L_E_METADATA");
        saveProperties(getUsername(), getPassword(), getUri(), getMetadataTable());
        setDefaultProperties();
    }

    // step 1: generate the user tree
    userTree = treeGeneration(userQuery, userTree);

    //step 2: decorrelate the user tree, if prompted
    if (decor) {
        userTree = treeDecorrelation(userTree);
    }

    if (merge) {
        // step 1: generate the mapping tree
        mappingTree = treeGeneration(viewQuery, mappingTree);

        // step 2: decorrelate the mapping tree
        if (decor) {
            mappingTree = treeDecorrelation(mappingTree);
        }

        // Step 3: merge the trees
        userTree = treeMerge(userTree, mappingTree);
    }

    if (rewrite) {
        //step 4: rewrite the tree
        userTree = treeRewrite(userTree);
    }

    if (schema) {
        //step 5: compute schema
        userTree = treeSchema(userTree);
    }

    if (cut) {
        //step 5.5: Tree cutting.
        userTree = treeCut(userTree);
    }

    if (debug)
        System.out.println(userTree.getBindingTable());

    if (sql) {
        //step 6: create sql
        userTree = treeSQLGen(userTree);
    }

    if (exec) {
        //step 7: execute the query over native
        userTree = treeExec(userTree);
    }

    System.out.println("\nFinished\n");

    // PRINT OUT THE TIME

    if (time)
        printTime("");

    System.exit(0);

}
/**
 * Output the time usage of each step in RainbowCore.
 * @param perfix The perfix for helping output.
 * @since 1.0
 */
public static void printTime(String prefix) {
    if (time) {
        System.out.println(prefix+"Total query generation time: " + timeGen + " ms");
        System.out.println(prefix+"Total query decorrelation time: " + timeDecor + " ms");
        if (merge) {
            System.out.println(prefix+"Merge time: " + timeMerge + " ms");
        }
        if (rewrite)
            System.out.println(prefix+"Rewrite time: " + timeRewrite + " ms");

        if (schema)
            System.out.println(prefix+"Schema time: " + timeSchema + " ms");

        if (cut)
            System.out.println(prefix+"Cut time: " + timeCut + " ms");
            
        if (sql)
            System.out.println(prefix+"SQL generation time: " + timeSQL + " ms");

        if (exec)
            System.out.println(prefix+"Execution time: " + timeExec + " ms");

        System.out.println(prefix+"Total time: " + timeTotal + " ms");
    }
	
}
/**
 * Get parameters from command line.
 * @param args java.lang.String[] The array containing all the parameters.
 * @param start The number of the first paramters.
 * @since 1.0
 */
public static void readParameters(String[] args, int start) {
	    for (int i = start; i < args.length; i++) {
        String flag = new String(args[i]);
        if (flag.equals("decor")) {
            decor = true;
        }
        if (flag.equals("newProp")) {
            newProp = true;
        }
        if (flag.equals("debug")) {
            debug = true;
        }
        if (flag.equals("exec")) {
            exec = true;
        }
        if (flag.equals("rewrite")) {
            rewrite = true;
        }
        if (flag.equals("schema")) {
            schema = true;
        }
        if (flag.equals("sql")) {
            sql = true;
        }
        if (flag.equals("time")) {
            time = true;
        }
        if (flag.equals("cut")) {
            cut = true;
        }
        if (flag.equals("nooutput")) {
            nooutput = true;
        }
    }
}
/**
 * Reads in the contents of a file and converts it to a string object.
 * @return String The content of the file.
 * @param file The name of file opening.
 * @since 1.0
 */
protected static String readQuery(String file) throws Exception {
    BufferedReader in =
        new BufferedReader(new InputStreamReader(new FileInputStream(file)));
    StringBuffer buf = new StringBuffer();
    String s;
    while ((s = in.readLine()) != null)
        buf.append(s + "\n");
    in.close();
    return buf.toString();
}
/**
 * Saves the user parameters to PROPERTIES_FILE_NAME 
 * @param userSave The username.
 * @param passwdSave The password.
 * @param uriSave The JDBC connecting string to database.
 * @since 1.0
 */
protected static void saveProperties(String userSave, String passwdSave, String uriSave) {
	defaultProperties = new Properties();
	try {
		defaultProperties.setProperty("username", userSave);
		defaultProperties.setProperty("password", passwdSave);
		defaultProperties.setProperty("uri", uriSave);
		defaultProperties.store(new FileOutputStream(PROPERTIES_FILE_NAME), "some header");	
	} catch (Exception e) {
		e.printStackTrace();
		return ;	
	}
	
	
}
/**
 * Saves the user parameters to PROPERTIES_FILE_NAME 
 * @param userSave The username.
 * @param passwdSave The password.
 * @param uriSave The JDBC connecting string to database.
 * @since 1.0
 */
 
protected static void saveProperties(String userSave, String passwdSave, String uriSave, String metaSave) {
	defaultProperties = new Properties();
	try {
		defaultProperties.setProperty("username", userSave);
		defaultProperties.setProperty("password", passwdSave);
		defaultProperties.setProperty("uri", uriSave);
		defaultProperties.setProperty("metadataTable", metaSave);
		defaultProperties.store(new FileOutputStream(PROPERTIES_FILE_NAME), "some header");	
	} catch (Exception e) {
		e.printStackTrace();
		return ;	
	}
	
	
}
/**
 * Set default properties into the PROPERTIES_FILE, which include username, password, uri and metadataTable.
 * @since 1.0
 */
public static void setDefaultProperties() {
    try {
        FileInputStream fileIn = new FileInputStream(PROPERTIES_FILE_NAME);
        //if the file cannot be found

        try {
	        defaultProperties.load(fileIn);
            setUsername(defaultProperties.getProperty("username"));
            setPassword(defaultProperties.getProperty("password"));
            setUri(defaultProperties.getProperty("uri"));
            setMetadataTable(defaultProperties.getProperty("metadataTable"));
        } catch (IOException e1) {
            System.err.println(e1.toString());
        }
    } catch (FileNotFoundException fnfe) {
        System.err.print("Properties file not found");
    }
}
/**
 * Set the metadataTable using given string.
 * @param meta The meta string.
 * @since 1.0
 */
public static void setMetadataTable(java.lang.String meta) {
	metadataTable = meta;
}
/**
 * Set the metadataTable using given string.
 * @param newPassword The new password.
 * @since 1.0
 */
public static void setPassword(java.lang.String newPassword) {
	password = newPassword;
}
/**
 * Set the metadataTable using given string.
 * @param newUri The JDBC connection string.
 * @since 1.0
 */
 
public static void setUri(java.lang.String newUri) {
	uri = newUri;
}
/**
 * Set the username using given string.
 * @param newUsername The new user name.
 * @since 1.0
 */

public static void setUsername(java.lang.String newUsername) {
	username = newUsername;
}
/**
 * Cut the unused operator within the tree.
 * @return XATTree The optimized tree after cutting.
 * @param debugCut The tree need to be optimized.
 * @since 1.0
 */
public static XATTree treeCut(XATTree treeCut) {
    /*************  Step 5: Compute The Schema **************/

    if (debug)
        System.out.println("**********Starting Tree Cleanup************");

    if (time) {
        timeBegin = System.currentTimeMillis();
    }

    TreeCleanupVisitor sc = new TreeCleanupVisitor();
    sc.visit(treeCut);

    timeCut = System.currentTimeMillis() - timeBegin;
    timeBegin = System.currentTimeMillis();
    timeTotal += timeCut;

    if (debug) {
        Debugger userDebug = new Debugger(treeCut);
        userDebug.printTree();
    }

    return treeCut;
}
/**
 * Get rid of recursive operator in the tree, such as groupBy, For.
 * @return XATTree The optimized tree after decorrelation.
 * @param treeDecor The tree need to be optimized.
 * @since 1.0
 */

public static XATTree treeDecorrelation(XATTree treeDecor) {
    /*************  Step 2: Decorrelate the Tree **************/
    timeBegin = System.currentTimeMillis();

    if (debug)
        System.out.println("*********Starting Query Decorrelation************");

    QueryDecorrelation qd1 = new QueryDecorrelation(treeDecor);
    XATTree newTree = qd1.getDecorTree();
    // keep the decorrelation time.
    timeTotal += System.currentTimeMillis() - timeBegin;
    if (timeDecor > 0) {
        timeDecor += System.currentTimeMillis() - timeBegin;
    } else {
        timeDecor = System.currentTimeMillis() - timeBegin;
    }

    if (debug) {
        Debugger debugDecor = new Debugger(newTree);
        debugDecor.printTree();
    }
    return newTree;
}
/**
 * Execute the tree.
 * @return XATTree The tree after execution.
 * @param debugExec The tree needs to be executed.
 * @since 1.0
 */

public static XATTree treeExec(XATTree treeExec) {
    /*************  Step 6: Execute The Query **************/
    if (debug)
        System.out.println("**********Starting Query Execution ************");

    if (time) {
        timeBegin = System.currentTimeMillis();
    }

    // make sure system abort when see exception during
    // the execution.
    System.setProperty("AbortOnException", "OK");

    ExecuteVisitor ev = new ExecuteVisitor(getUri(), getUsername(), getPassword());
    if (debug)
        System.setProperty("DEBUG_ExecuteVisitor", "OK");
    ev.visit(treeExec);
    XATTable resultTable =
        (XATTable) treeExec.getRoot().getOperator().getOutputDataSource();

    if (time) {
        timeExec = System.currentTimeMillis() - timeBegin;
        timeTotal += timeExec;
    }

    if (!nooutput)
        System.out.println("************This is the XML Document***********\n");
    try {
        if (!nooutput)
            System.out.println(
                resultTable.getColumn(resultTable.getNumberOfColumns()).toString());
    } catch (NullPointerException npe) {
        npe.printStackTrace(System.err);
    } catch (XATTableOutOfRangeException xat) {
        System.out.println(xat);
    }

    return treeExec;
}
/**
 * Generate XAT for given query
 * @return XATTree The generated XAT tree.
 * @param genTree The generated XAT tree.
 * @param query The XQuery for tree generation.
 * @since 1.0
 */
public static XATTree treeGeneration(String query, XATTree genTree) {

    // keep the start time.
    timeBegin = System.currentTimeMillis();

    try {
        QuiltParser parser = new QuiltParser();
        QuiltQuery q = null;
        q = parser.parseQuery(query);
        ParserTraverse ptU = new ParserTraverse(debug);

        if (debug)
            System.out.println("*************Generating the Tree****************");
        /*************  Step 1: Generate the Tree **************/
        ptU.visit(q);
        genTree = ptU.getTree();

        Debugger debugGen = new Debugger(genTree);
        if (debug) {
            debugGen.printTree();
        }

        if (debug) {
            System.setProperty("DEBUG_RewriteUtilities", "OK");
        }

        /** cleanup the generated tree **/
        // Use System Property: CLEANXAT instead.

        // keep the generation time.
        timeTotal += System.currentTimeMillis() - timeBegin;
        if (timeGen > 0) {
            timeGen += System.currentTimeMillis() - timeBegin;
        } else {
            timeGen = System.currentTimeMillis() - timeBegin;
        }

        if (debug) {
            debugGen.printTree();
        }
    } catch (Exception e) {
        e.printStackTrace();
    }

    return genTree;
}
/**
 * Merge the two XATs.
 * @return XATTree The merged XAT tree.
 * @param userTree The generated user XAT tree.
 * @param query The generated view XATTree.
 * @since 1.0
 */

public static XATTree treeMerge(XATTree userTree, XATTree viewTree) {

    /*********** Now merge the trees *******/
    timeBegin = System.currentTimeMillis();

    if (debug)
        System.out.println("**********Now merging the trees************");

    MergeVisitor m1 = new MergeVisitor(userTree, viewTree);
    m1.mergeTrees();
    /*
    userTree.setBindingTable(
        //m1.mergeBindingTables(
            userTree.getBindingTable()//,
            //viewTree.getBindingTable()));
    );
    */
    timeMerge = System.currentTimeMillis() - timeBegin;
    timeBegin = System.currentTimeMillis();
    timeTotal += timeMerge;

    if (debug) {
        Debugger userDebug = new Debugger(userTree);
        userDebug.printTree();
    }

    return userTree;
}
/**
 * Rewrite the XAT for optimization.
 * @return XATTree The optimized XAT tree.
 * @param treeRewrite The tree need to be optimized.
 * @since 1.0
 */
public static XATTree treeRewrite(XATTree treeRewrite) {

    Debugger userDebug = new Debugger(treeRewrite);
    timeBegin = System.currentTimeMillis();

    if (sql) {
        if (debug)
            System.out.println(
                "**********Now swapping position/order functions ************");
        //First, remove any odd nestings of groupbys
        Vector order = new Vector();
        Vector classList = new Vector();
        order.addElement(
            new String("edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupByImp"));
        classList.addElement(
            new String("edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.MergeOperatorRules"));
        ReWriteRules rules = new ReWriteRules(classList);
        rules.setVariableTable(treeRewrite.getBindingTable());
        ReWriteTraversal traverse = new ReWriteTraversal(treeRewrite, rules, order);

        if (debug)
            System.setProperty("DEBUG_ReWriteTraversal", "OK");

        traverse.doPushdown(ReWriteTraversal.DOWN);

        //Second do the actual position rewrites
        Vector classListOrder = new Vector();
        classListOrder.addElement(
            "edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ReWritePositionRules");
        //instantiate the rules class 
        ReWriteRules rulesOrder = new ReWriteRules(classListOrder);

        //now make the order of rules to look at
        Vector orderPos = new Vector();
        orderPos.addElement(
            "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupByImp");
        orderPos.addElement(
            "edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.FunctionImp");

        //set the variable table in the rules class to be the hashtable from PT
        rulesOrder.setVariableTable(treeRewrite.getBindingTable());

        //create the traversal strategy
        ReWriteTraversal traverseOrder =
            new ReWriteTraversal(treeRewrite, rulesOrder, orderPos);

        if (debug)
            System.setProperty("DEBUG_ReWriteTraversal", "OK");
        traverseOrder.doPushdown(traverseOrder.DOWN);

        if (debug) {
            userDebug.printTree();
        }

    }

    //change all the duplicate navigates into renames
    //if(debug)
    //	System.out.println("********** Navigation Tagger Cancel OUT*************");
    if (debug)
        System.out.println("**********Now merging Navigates and Taggers************");

    BindingTable mergedTable =
        (new BindingTableComputer()).computeBindingTable(treeRewrite);
    treeRewrite.setBindingTable(mergedTable);
    NavTaggerCancelout.cancelOut(treeRewrite);

    //first merge the Navigates
    //first create a vector of the rule classes to look at
    Vector classList = new Vector();
    classList.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.MergeOperatorRules");
    //instantiate the rules class 
    ReWriteRules rules = new ReWriteRules(classList);

    //now make the order of rules to look at
    Vector order = new Vector();
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollectionImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnestImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.DistinctImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.TaggerImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupByImp");

    //set the variable table in the rules class to be the hashtable from PT
    rules.setVariableTable(treeRewrite.getBindingTable());

    //create the traversal strategy
    ReWriteTraversal traverse = new ReWriteTraversal(treeRewrite, rules, order);

    if (debug)
        System.setProperty("DEBUG_ReWriteTraversal", "OK");
    traverse.doPushdown(traverse.DOWN);

    if (debug) {
        userDebug.printTree();
    }

    if (debug)
        System.out.println("**********Now rewriting the trees************");
    ///////////////////////////// Now rewriting the rest of the tree ///////////			         

    //first create a vector of the rule classes to look at
    classList = new Vector();
    String s1 =
        new String("edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ComputationPushdownRules");
    classList.addElement(s1);

    //instantiate the rules class
    rules = new ReWriteRules(classList);

    //now make the order of Operator types to look at
    order = new Vector();
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.NameColumnImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollectionImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnestImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollectionImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.NameColumnImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.SelectImp");
    //set the variable table in the rules class to be the hashtable from PT
    rules.setVariableTable(treeRewrite.getBindingTable());

    if (debug)
        System.setProperty("DEBUG_ReWriteTraversal", "OK");
    //create the traversal strategy
    traverse = new ReWriteTraversal(treeRewrite, rules, order);

    traverse.doPushdown(traverse.DOWN);
    timeRewrite = System.currentTimeMillis() - timeBegin;
    timeTotal += timeRewrite;
    if (debug) {
        userDebug.printTree();
    }

    return treeRewrite;
}
/**
 * Do schema cleanup for given XAT.
 * @return XATTree The optimized XATTree.
 * @param treeSchema Cleanup the XATTree.
 * @since 1.0
 */
public static XATTree treeSchema(XATTree treeSchema) {
    /*************  Step 5: Compute The Schema **************/

    if (debug)
        System.out.println("**********Starting Schema Computation************");

    if (time) {
        timeBegin = System.currentTimeMillis();
    }

    SchemaCleanupVisitor sc = new SchemaCleanupVisitor();
    sc.visit(treeSchema);

    timeSchema = System.currentTimeMillis() - timeBegin;
    timeBegin = System.currentTimeMillis();
    timeTotal += timeSchema;

    if (debug) {
        Debugger userDebug = new Debugger(treeSchema);
        userDebug.printTree();
    }

    return treeSchema;
}
/**
 * Generate SQL for "relational" algebra part within given XAT.
 * @return XATTree The tree after the SQL generation step.
 * @param treeSQL The XATTree used for SQL generation.
 * @since 1.0
 */
public static XATTree treeSQLGen(XATTree treeSQL) {
    /*************  Step 6: Generate the SQL **************/
    //pass in the binding hashtable to the sql generator

    if (time) {
        timeBegin = System.currentTimeMillis();
    }
	Debugger userDebug = new Debugger(treeSQL);
	
    QueryGenerator sqlGen = new QueryGenerator(treeSQL.getBindingTable(), debug);
    try{
		sqlGen.generateQuery(treeSQL, sqlGen.FULL, QueryTypes.SQL);
    }catch (UnknownQueryType uqt){
		System.out.println(uqt);
    }
    timeSQL = System.currentTimeMillis() - timeBegin;
    timeBegin = System.currentTimeMillis();
    timeTotal += timeSQL;
    if (debug) {
        userDebug.printTree();
    }

    return treeSQL;
}
}
